<?php

class Am_Paysystem_Cashbill extends Am_Paysystem_Abstract
{
    const PLUGIN_STATUS = self::STATUS_BETA;
    const PLUGIN_REVISION = '1.1.0';
    
    protected $defaultTitle = 'CashBill';
    protected $defaultDescription = 'Płatności CashBill to usługa agregująca wszystkie dostępne na rynku finansowym instrumenty płatnicze. Integrując nasz pakiet płatności ze swoim serwisem zapewniasz, że każdy klient będzie mógł dokonać płatności elektronicznej.';

    const ACTION_URL = "https://pay.cashbill.pl/form/pay.php";

    protected $transaction;

    public function _initSetupForm(Am_Form_Setup $form)
    {
        $form->addText('ipp', array('size' => 50))
            ->setLabel('Identyfikator Punktu Płatności :');
        $form->addText('kpp', array('size' => 50))
            ->setLabel('Klucz Punktu Płatności :');
    }

    public function isConfigured()
    {
        return $this->getConfig('ipp') > '';
    }


    public function _process(Invoice $invoice, Am_Request $request, Am_Paysystem_Result $result)
    {
        $action = new Am_Paysystem_Action_Form(self::ACTION_URL);

        $action->service = $this->getConfig('ipp');
        $action->amount = $invoice->first_total;
        $action->currency = "PLN";
        $action->desc = $invoice->getLineDescription();
        $action->lang = "";
        $action->userdata = $invoice->public_id;
        $action->ref = "aMember";
        $action->forname = $invoice->getFirstName();
        $action->surname = $invoice->getLastName();
        $action->email = $invoice->getEmail();
        $action->tel = "";
        $action->street = $invoice->getStreet();
        $action->street_n1 = "";
        $action->street_n2 = "";
        $action->city = $invoice->getCity();
        $action->postcode = $invoice->getZip();
        $action->country = $this->getConfig('lang');

        $action->sign = md5 ( $action->service . '|' . $action->amount . '|' . $action->currency . '|' . $action->desc . '|' .
        $action->lang . '|' . $action->userdata . '|' . $action->ref . '|' . $action->forname . '|' . $action->surname . '|' .
        $action->email . '|' . $action->tel . '|' . $action->street . '|' . $action->street_n1 . '|' . $action->street_n2 . '|' .
        $action->city . '|' . $action->postcode . '|' .$action->country . '|' . $this->getConfig('kpp') );
        
        $result->setAction($action);
    }
    
    public function getSupportedCurrencies()
    {
        return array('PLN');
    }

    public function createTransaction(Am_Request $request, Zend_Controller_Response_Http $response, array $invokeArgs)
    {
        return new Am_Paysystem_Transaction_CashBill($this, $request, $response, $invokeArgs);
    }

    public function getRecurringType()
    {
        return self::REPORTS_REBILL;
    }

    function getReadme()
    {
        $rootURL = $this->getDi()->config->get('root_url');

        return <<<CUT
<b>Płatności CashBill</b>

1. Przejdź do panelu aMemeber -> Setup/Configuration -> Plugins i włącz plugin CashBill.

2. Przejdź do panelu aMemeber -> Setup/Configuration -> CashBill i wpisz dane założonego punktu płatności znajdujące się w panelu CashBill.

3. Zainicjuj testową płatność i przetestuj działanie płatności CashBill.
CUT;
    }

    function directAction(Am_Request $request, Zend_Controller_Response_Http $response, array $invokeArgs){

        try
        {
            return parent::directAction($request, $response, $invokeArgs);
        }
        catch (Exception $e)
        {
            $this->getDi()->errorLogTable->log($e);
            print "ERROR";
            exit;
        }
    }


    function isRefundable(InvoicePayment $payment)
    {
        return false;
    }
    
}

class Am_Paysystem_Transaction_Cashbill extends Am_Paysystem_Transaction_Incoming
{
    const ACTION_ADD = 'user.add';
    const ACTION_REBILL = 'rebill';
    const ACTION_DELETE = 'user.delete';

    public function findInvoiceId(){
        return $this->transaction->userdata;
    }

    public function genereateSignature($service, $orderid, $userdata, $amount, $status)
    {
        return md5($service . $orderid . $amount . $userdata . $status . $this->plugin->getConfig("kpp"));
    }

    public function checkSignature($service, $orderid, $userdata, $amount, $status, $key, $sign){
        if(md5($service . $orderid . $amount . $userdata . $status . $key) === $sign){
            return true;
        }

        if(md5($service .'|'. $orderid .'|'. $amount .'|'. $userdata .'|'. $status .'|'. $key) === $sign){
            return true;
        }

        return false;
    }
    
    public function getUniqId()
    {
        return $this->transaction->userdata;
    }

    public function validateSource()
    {
        if(!$this->request->get("service") || !$this->request->get("orderid") || !$this->request->get("amount") || !$this->request->get("userdata") ||  !$this->request->get("status") ||  !$this->request->get("sign"))
            throw new Am_Exception_Paysystem_TransactionInvalid("Empty Parametrs!");

        if($this->request->get("service") != $this->plugin->getConfig("ipp"))
            throw new Am_Exception_Paysystem_TransactionInvalid("WRONG SERVICE!");

        if($this->checkSignature($this->request->get("service"), $this->request->get("orderid"), $this->request->get("userdata"), $this->request->get("amount"), $this->request->get("status"), $this->plugin->getConfig("kpp"), $this->request->get("sign")))
            throw new Am_Exception_Paysystem_TransactionInvalid("WRONG SIGNATURE!");

        $return = (object) array(
            'service'=>$this->request->get("service"),
            'orderid'=>$this->request->get("orderid"),
            'amount'=>$this->request->get("amount"),
            'userdata'=>$this->request->get("userdata"),
            'status'=>$this->request->get("status"));
        
        $this->transaction = $return;
        
        return true;
    }


    public function validateStatus()
    {
        return true;
    }

    public function validateTerms()
    {
        return true;
    }
    
    public function processValidated(){

        switch($this->transaction->status){
            case "ok" :
                if($this->invoice->getStatus() == 0)
                      $this->invoice->addPayment($this);
                break;
            case "err" :
               $this->invoice->stopAccess($this);
                break;
        }
        print "OK";

    }
    
}